/*
 * ----------------------------------------------------------------------------------------------
 *			Fichier entete global pour l'assembleur.
 *			Dfinit les types, units lexicales, etc...
 *
 * 	Auteur 	: 	FAVARD Laurent
 *	Fichier	:	ASSEMBLE.H
 *	Pays   	: 	FRANCE
 *	Date   	: 	20 Mars 1994
 *	Revision: 	19 November 1996
 *	Version : 	2.0
 *	Machine : 	ATARI Falcon030 TOS 4.2
 *
 * ---------------------------------------------------------------------------------------------- 
 */

#ifndef __ASSEMBLE__
#define __ASSEMBLE__ __ASSEMBLE__

/*
 * ----------------------------------------------------------------------------------------------
 *	Processeurs supports: Ces valeurs sont prcises par la directive Opt MC68HC11, 
 *	qui permettent d'identifer quel jeux d'insructions et codes machines  utiliser pour 
 *	la gnration de codes.
 * ----------------------------------------------------------------------------------------------
 */

/*	Processors list supported */

#define	PROCESSOR_6805		"MC6805"
#define	PROCESSOR_68HC11	"MC68HC11"

/*
 * ----------------------------------------------------------------------------------------------
 *	Modes d'adressages (Leurs syntaxe typique) supports
 * ----------------------------------------------------------------------------------------------
 */
 
#define	MODE_INHERANT			"INHERANT"		/* no operand */
#define	MODE_IMMEDIAT8			"IMMEDIAT8"		/* 8 bits immediat */	
#define	MODE_IMMEDIAT16			"IMMEDIAT16"		/* 16 bits immediat */
#define	MODE_RELATIF			"RELATIF"		/* 8 bits signed */
#define	MODE_DIRECT			"DIRECT"		/* Address on 8 bits */
#define	MODE_EXTEND			"EXTEND"		/* Address on 16 bits */

#define	MODE_INDEX_NULL			"INDEXED_NULL"		/* Null offset => special opcode */
#define	MODE_INDEX_8			"INDEXED_8"		/* 8 bits offset */
#define	MODE_INDEX_16			"INDEXED_16"		/* 16 bits offset */

#define	MODE_BSC			"BITSC"			/* Bits Set Clear */
#define	MODE_BTB			"BITTB"			/* Bits Test and Branch */


/*
 * ----------------------------------------------------------------------------------------------
 *				Dfinitions diverses
 * ----------------------------------------------------------------------------------------------
 */

#define TRUE			1
#define FALSE			0

#define TAILLE4OCTETS		4			/* mot 16 bits = 4 digits */
#define TAILLE16BITS		16			/* mot 16 bits = 4 digits */
#define TAILLE_ID		26			/* len(ID) and len(STRING) */
#define	TAILLE_ID_PROCESSOR	50
#define	TAILLE_STRING		256

#define	LEN_INSTRUCTION		10
#define	LEN_ADRESSAGE		20

#define STARTCODEDEFAULT 	0x00			/* dbut programme user */

#define	BUFFER_SIZE		65535L			/* Tampon d'assemblage */

#define	SIZE_8_BITS		1			/* Donnees 1 octets */
#define	SIZE_16_BITS		2			/* Donnees 2 octets */

/*
 * ----------------------------------------------------------------------------------------------
 *			Dfinition des units lexicales reconnues
 * ----------------------------------------------------------------------------------------------
 */

#define RIEN		0	/* rien du tout */
#define ID		1	/* Identificateurs */
#define NB		2	/* Nombre */
#define FDL		3	/* Fin De Ligne */
#define FDF		4	/* Fin De Fichier */
#define INST		5	/* instruction processeur 68705 */
#define LEXERREUR	6	/* Erreur d'analyse lexicale */
#define VIRG		7	/* La , */
#define DIESE		8	/* Le # */
#define PRCENT		9	/* Le % */
#define DOLLAR		10	/* Le $ */
#define UNSCORE		11	/* Le _ */
#define	DBP		12	/* Le : */
#define	PARAMDIRECTIVE	13	/* Est un paramtre de directive, ex: OPT MC6805 */
#define	STRING		14	/* Chaine de caracteres ex: dcb "toto" */

#define EQU		20	/* Directive */
#define	ORG		21	/* Directive */
#define END		22	/* Directive */
#define DCB		23	/* Directive */
#define DSB		24	/* Directive */
#define TEXT		25	/* Directive */
#define DATA		26	/* Directive */
#define OPT		27	/* Directive */
#define DCW		28	/* Directive */
#define DSW		29	/* Directive */


/*
 * ----------------------------------------------------------------------------------------------
 *				 Dfinitions des types
 * ----------------------------------------------------------------------------------------------
 */

/* Contient la valeur lexicale d'une unit lexicale: Soit un lexeme texte
   pour un ID,INST,..., soit une valeur numrique pour un NB.
*/
typedef struct {
		 unsigned long	numerique;
		 char 		lexeme[TAILLE_STRING];
	       } Type_ValLex;

extern 	FILE	*fichier;			/* Fichier d'entre du texte source */
extern 	FILE	*FileLST;			/* source+assembl */
extern 	char	*TamponPRG;
extern 	int	TailleTamponPRG;
extern 	int	ListingScreen;
extern	int	C_STRING;
extern	void	EcrireMessage(char *Message);

#endif
