/*
 * ----------------------------------------------------------------------------------------------
 *				GESTION DES SPECIFICTES DU MC 6805.
 *
 *				Jeux d'instructions code op.
 *				Prototypes de procdures ddies (cf: MC6805.C)
 *				
 * 	Auteur 	: 	FAVARD Laurent
 *	Fichier	: 	MC6805.H
 *	Pays   	: 	FRANCE
 *	Date   	: 	20 March 1994
 *	Revision: 	05 October 1996
 *	Version : 	1.0
 *	Release : 	1.0
 *	Machine : 	ATARI Falcon030 TOS 4.2
 *
 * ----------------------------------------------------------------------------------------------
 */

#ifndef __MC6805__
#define __MC6805__	__MC6805__

/*
 * ----------------------------------------------------------------------------------------------
 *			prototypes des procedures dediees
 * ----------------------------------------------------------------------------------------------
 */


extern int BSC6805(char *hinst,int valeur,int *CodeHexa);
extern int BTB6805(char *hinst,int valeur,int *CodeHexa);


/*
 * ----------------------------------------------------------------------------------------------
 *			Implementation du jeux d'instructions
 * ----------------------------------------------------------------------------------------------
 */

#define	MAX_INST05	200		/* Number of instructions defined */

/*
 * ----------------------------------------------------------------------------------------------
 *			Definition du type d'une entree dans une table de codes
 * ----------------------------------------------------------------------------------------------
 */

#ifndef	__TYPE_TABLE__
#define	__TYPE_TABLE__	__TYPE_TABLE__
 
typedef struct {
		 char Inst[LEN_INSTRUCTION];			/* Mnemonique Instruction */
		 char ModeAdressage[LEN_ADRESSAGE];		/* Addressing Mode */
		 int  CodeMachine;				/* Machine Code */
	       } Type_CodeMachine;

#endif

/*
 * ----------------------------------------------------------------------------------------------
 *			Tables des codes machines du MC 6805
 * ----------------------------------------------------------------------------------------------
 */

#ifndef	__TABLE6805__
#define	__TABLE6805__	__TABLE6805__

static Type_CodeMachine Table6805[MAX_INST05]=
				   {

/*
 *	Mode INHERANT
 */			
	       			    {"ASRA","INHERANT",0x47},{"ASRX","INHERANT",0x57},
				    {"ASLA","INHERANT",0x48},{"ASLX","INHERANT",0x58},
				    {"CLRA","INHERANT",0x4F},{"CLRX","INHERANT",0x5F},
				    {"COMA","INHERANT",0x43},{"COMX","INHERANT",0x53},
				    {"DECA","INHERANT",0x4A},{"DECX","INHERANT",0x5A},
				    {"INCA","INHERANT",0x4C},{"INCX","INHERANT",0x5C},
				    {"LSLA","INHERANT",0x48},{"LSLX","INHERANT",0x58},
				    {"LSRA","INHERANT",0x44},{"LSRX","INHERANT",0x54},
				    {"NEGA","INHERANT",0x40},{"NEGX","INHERANT",0x50},
				    {"ROLA","INHERANT",0x49},{"ROLX","INHERANT",0x59},
				    {"RORA","INHERANT",0x46},{"RORX","INHERANT",0x56},
				    
				    {"CLI", "INHERANT",0x9A},{"CLC", "INHERANT",0x98},
				    {"NOP", "INHERANT",0x9D},{"RTI", "INHERANT",0x80},
				    {"RSP", "INHERANT",0x9C},{"RTS", "INHERANT",0x81},
				    {"STOP","INHERANT",0x8E},{"SWI", "INHERANT",0x83},
				    {"SEI", "INHERANT",0x9B},{"SEC", "INHERANT",0x99},
				    {"TSTA","INHERANT",0x4D},{"TSTX","INHERANT",0x5D},
				    {"TAX", "INHERANT",0x97},{"TXA", "INHERANT",0x9F},
				    {"WAIT","INHERANT",0x8F},

/*
 *	Mode Immediat with 8 bits operand
 */

				    {"ADD", "IMMEDIAT8",0xAB},{"ADC", "IMMEDIAT8",0xA9},
				    {"AND", "IMMEDIAT8",0xA4},{"BIT", "IMMEDIAT8",0xA5},
				    {"CPX", "IMMEDIAT8",0xA3},{"CMP", "IMMEDIAT8",0xA1},
				    {"EOR", "IMMEDIAT8",0xA8},{"LDA", "IMMEDIAT8",0xA6},
				    {"LDX", "IMMEDIAT8",0xAE},{"ORA", "IMMEDIAT8",0xAA},
				    {"SUB", "IMMEDIAT8",0xA0},{"SBC", "IMMEDIAT8",0xA2},
/*
 *	Mode Immediat with 16 bits operand
 */


/*
 *	Mode Relative with 8 bits signed offset
 */

				    {"BRA", "RELATIF",0x20},{"BRN", "RELATIF",0x21},
				    {"BHI", "RELATIF",0x22},{"BLS", "RELATIF",0x23},
				    {"BCC", "RELATIF",0x24},{"BHS", "RELATIF",0x24},
				    {"BCS", "RELATIF",0x25},{"BLO", "RELATIF",0x25},
				    {"BNE", "RELATIF",0x26},{"BEQ", "RELATIF",0x27},
				    {"BHCC","RELATIF",0x28},{"BHCS","RELATIF",0x29},
				    {"BPL", "RELATIF",0x2A},{"BMI", "RELATIF",0x2B},
				    {"BMC", "RELATIF",0x2C},{"BMS", "RELATIF",0x2D},
				    {"BIL", "RELATIF",0x2E},{"BIH", "RELATIF",0x2F},
				    {"BSR", "RELATIF",0xAD},

/*
 *	Mode Direct with 8 bits effective address
 */

				    {"ASL","DIRECT",0x38},{"ASR","DIRECT",0x37},
				    {"ADD","DIRECT",0xBB},
				    {"ADC","DIRECT",0xB9},{"AND","DIRECT",0xB4},
				    {"BIT","DIRECT",0xB5},{"CMP","DIRECT",0xB1},
				    {"CPX","DIRECT",0xB3},{"CLR","DIRECT",0x3F},
				    {"COM","DIRECT",0x33},{"DEC","DIRECT",0x3A},
				    {"EOR","DIRECT",0xB8},{"INC","DIRECT",0x3C},
				    {"JMP","DIRECT",0xBC},{"JSR","DIRECT",0xBD},
				    {"LSL","DIRECT",0x38},{"LDA","DIRECT",0xB6},
				    {"LDX","DIRECT",0xBE},{"LSR","DIRECT",0x34},
				    {"NEG","DIRECT",0x30},{"ORA","DIRECT",0xBA},
				    {"ROL","DIRECT",0x39},{"ROR","DIRECT",0x36},
				    {"STA","DIRECT",0xB7},{"STX","DIRECT",0xBF},
				    {"SUB","DIRECT",0xB0},{"SBC","DIRECT",0xB2},
				    {"TST","DIRECT",0x3D},

/*
 *	Mode Extend with 16 bits effective address
 */

				    {"ADD","EXTEND",0xCB},{"ADC","EXTEND",0xC9},
				    {"AND","EXTEND",0xC4},{"BIT","EXTEND",0xC5},
				    {"CMP","EXTEND",0xC1},{"CPX","EXTEND",0xC3},
				    {"EOR","EXTEND",0xC8},
				    {"JMP","EXTEND",0xCC},{"JSR","EXTEND",0xCD},
				    {"LDA","EXTEND",0xC6},
				    {"LDX","EXTEND",0xCE},{"ORA","EXTEND",0xCA},
				    {"STA","EXTEND",0xC7},{"STX","EXTEND",0xCF},
				    {"SUB","EXTEND",0xC0},{"SBC","EXTEND",0xC2},				    
	  
/*
 *	Mode INDEXED null offset: Agressiv will place the _X for index register X
 *				  ( Or _Y}, ... for any index register asked 
 */

				    {"INC","INDEXED_NULL_X",0x7C},{"DEC","INDEXED_NULL_X",0x7A},
				    {"CLR","INDEXED_NULL_X",0x7F},{"COM","INDEXED_NULL_X",0x73},
				    {"NEG","INDEXED_NULL_X",0x70},{"ROL","INDEXED_NULL_X",0x79},
				    {"ROR","INDEXED_NULL_X",0x76},{"LSL","INDEXED_NULL_X",0x78},
				    {"LSR","INDEXED_NULL_X",0x74},{"ASR","INDEXED_NULL_X",0x77},
				    {"ASL","INDEXED_NULL_X",0x78},{"TST","INDEXED_NULL_X",0x7D},
				    {"LDA","INDEXED_NULL_X",0xF6},{"LDX","INDEXED_NULL_X",0xFE},
				    {"STA","INDEXED_NULL_X",0xF7},{"STX","INDEXED_NULL_X",0xFF},
				    {"ADD","INDEXED_NULL_X",0xFB},{"ADC","INDEXED_NULL_X",0xF9},
				    {"SUB","INDEXED_NULL_X",0xF0},{"SBC","INDEXED_NULL_X",0xF2},
				    {"AND","INDEXED_NULL_X",0xF4},{"ORA","INDEXED_NULL_X",0xFA},
				    {"EOR","INDEXED_NULL_X",0xF8},{"CMP","INDEXED_NULL_X",0xF1},
				    {"CPX","INDEXED_NULL_X",0xF3},{"BIT","INDEXED_NULL_X",0xF5},
				    {"JMP","INDEXED_NULL_X",0xFC},{"JSR","INDEXED_NULL_X",0xFD},
/*
 *	Mode Indexed 8 bits offset
 */

				    {"INC","INDEXED_8_X",0x6C},{"DEC","INDEXED_8_X",0x6A},
				    {"CLR","INDEXED_8_X",0x6F},{"COM","INDEXED_8_X",0x63},
				    {"NEG","INDEXED_8_X",0x60},{"ROL","INDEXED_8_X",0x69},
				    {"ROR","INDEXED_8_X",0x66},{"LSL","INDEXED_8_X",0x68},
				    {"LSR","INDEXED_8_X",0x64},{"ASR","INDEXED_8_X",0x67},
				    {"ASL","INDEXED_8_X",0x68},{"TST","INDEXED_8_X",0x6D},
				    {"LDA","INDEXED_8_X",0xE6},{"LDX","INDEXED_8_X",0xEE},
				    {"STA","INDEXED_8_X",0xE7},{"STX","INDEXED_8_X",0xEF},
				    {"ADD","INDEXED_8_X",0xEB},{"ADC","INDEXED_8_X",0xE9},
				    {"SUB","INDEXED_8_X",0xE0},{"SBC","INDEXED_8_X",0xE2},
				    {"AND","INDEXED_8_X",0xE4},{"ORA","INDEXED_8_X",0xEA},
				    {"EOR","INDEXED_8_X",0xE8},{"CMP","INDEXED_8_X",0xE1},
				    {"CPX","INDEXED_8_X",0xE3},{"BIT","INDEXED_8_X",0xE5},
				    {"JMP","INDEXED_8_X",0xEC},{"JSR","INDEXED_8_X",0xED},

/*
 *	Indexed 16 bits offset
 */

				    {"LDA","INDEXED_16_X",0xD6},{"LDX","INDEXED_16_X",0xDE},
				    {"STA","INDEXED_16_X",0xD7},{"STX","INDEXED_16_X",0xDF},
				    {"ADD","INDEXED_16_X",0xDB},{"ADC","INDEXED_16_X",0xD9},
				    {"SUB","INDEXED_16_X",0xD0},{"SBC","INDEXED_16_X",0xD2},
				    {"AND","INDEXED_16_X",0xD4},{"ORA","INDEXED_16_X",0xDA},
				    {"EOR","INDEXED_16_X",0xD8},{"CMP","INDEXED_16_X",0xD1},
				    {"CPX","INDEXED_16_X",0xD3},{"BIT","INDEXED_16_X",0xD5},
				    {"JMP","INDEXED_16_X",0xDC},{"JSR","INDEXED_16_X",0xDD},

/*
 *	Bits Set and Clear AND Bits Test and Branch
 */


				    {"BSET", "BITSC",0x00},{"BCLR", "BITSC",0x00},
				    {"BRSET","BITTB",0x00},{"BRCLR","BITTB",0x00},

				    {"","",0x00} };

#endif

#endif
