*******************************************************************************************
*
*		Memory Test Program for the 68HC11F1. 
*		(Sources file for CAM assembler.)
*
*******************************************************************************************
*		Ce programme est telecharge en RAM interne du HC11, puis sous la demande du host
*		effectue un test memoire complet de la RAM 32Ko et 8Ko.
*
*		FAVARD Laurent, 1996-97 ATARI Computers range
*******************************************************************************************
*
*	File	:	TESTMEM.S
*	Target	:	MC 68HC11F1
*	Date	:	29 April 1997
*	Release	:	04 May 1997
*	Author	:	FAVARD Laurent alias Funship
*
*******************************************************************************************

				OPT		MC68HC11					;	Target processor

*
*	Values for the system configuration
*

VAL_HPRIO		Equ		%00100110					;	BootRom disabled, expanded mode
VAL_CONFIG		Equ		%11110101					;	COP disabled, EEPROM at $FE00 and enabled
VAL_INIT		Equ		%11111110					;	Internal RAM at $F000 and regs at $E000
VAL_OPTION		Equ		%00000000
VAL_OPT2		Equ		%00000000
VAL_CSGADR		Equ		%00000000					;	RAM 32Kb base is A15 = 0
VAL_CSGSIZ		Equ		%00000001					;	32 Kb, CSGEN active low and valid during E
VAL_CSCTL		Equ		%00001011					;	CSGEN > CSPROG and CSPROG disable
VAL_CSSTRH		Equ		%01010101					;	One cycle

*
*	Base and relative address registers for HC11
*

;REGISTERS		Equ		$E000						;	Base address register for the new mapping
REGISTERS		Equ		$1000						;	Base address register for the new mapping

BAUD			Equ		$002B
SCCR1			Equ		$002C
SCCR2			Equ		$002D
SCSR			Equ		$002E
SCDR			Equ		$002F

OPT2			Equ		$0038
OPTION			Equ		$0039
HPRIO			Equ		$003C
INIT			Equ		$103D						;	This address is DIRECT !
CONFIG			Equ		$003F
CSSTRH			Equ		$005C
CSCTL			Equ		$005D
CSGADR			Equ		$005E
CSGSIZ			Equ		$005F

*
*	Typical HC11 data constants
*

SCI_BPS19200	Equ		%00100000					;	SCI speed rate at 19200 bps
SCI_BPS4800		Equ		%00100010					;	SCI speed rate at 4800 bps
SCI_BPS300		Equ		%00100110					;	SCI speed rate at 4800 bps

SCI_FORMAT		Equ		%00000000					;	8 Bits, NO WAKE UP, 1 stop bit
SCI_CONTROL		Equ		%00001100					;	No Interrupt, TxD and RxD Enable

TDRE			Equ		%10000000					;	Transmit Data Register Empty
TC				Equ		%01000000
RDRF			Equ		%00100000					;	Receive Data Register Full

CCR_BITZ		Equ		%11111011					;	Bit Z du CCR

*
*	User data constants
*

SIZE_32K		Equ		15							;	Size to test
START_32K		Equ		$0300						;	Start memory address

SIZE_8K			Equ		100							;	Size to test
START_8K		Equ		$C000						;	Start memory address

END_STRING		Equ		$00							;	End of string to send
INTERROGATION	Equ		$3F							;	'?' character
GO				Equ		$47							;	'G'	character

DATA_ECRIT		Equ		$AA							;	Data write in memory

CR				Equ		$0D

*******************************************************************************************
*				Start of text code
*******************************************************************************************


				TEXT		
				ORG		$0000						;	Start at the bottom of RAM memory

;
;	Special hardware settings
;

Main:		;	ldaa	#VAL_INIT					;	New memory and registers map
			;	staa	INIT						;	Now, the base registers is changed

				ldx		#REGISTERS
	
				ldaa	#VAL_CONFIG
				staa	CONFIG,X

				ldaa	#VAL_OPTION
				staa	OPTION,X

				ldaa	#VAL_OPT2
				staa	OPT2,X

				ldaa	#VAL_HPRIO					;	Go to extended mode
				staa	HPRIO,X

				ldaa	#VAL_CSGADR
				staa	CSGADR,X

				ldaa	#VAL_CSGSIZ
				staa	CSGSIZ,X

				ldaa	#VAL_CSCTL
				staa	CSCTL,X

				ldaa	#VAL_CSSTRH
				staa	CSSTRH,X

;
;	Now we can start really the program
;

				lds		#stack						;	Initialise stack pointer
				
				jsr		InitSCI						;	Initialise SCI
				ldx		#Welcome					;	32 Kb memory test
				jsr		SendStringSCI

;
;	Main loop command
;

Command:		jsr		WaitByteSCI					;	A <= a byte from the serial port

				cmpa	#INTERROGATION
				beq		Menu1
			
				cmpa	#GO	
				beq		Menu2

				ldx		#Text4
				jsr		SendStringSCI

				bra		Command						; go to the main loop

;
;	About this program
;

Menu1:
				ldx		#Welcome					;	32 Kb memory test
				jsr		SendStringSCI
				bra		Command						; return to the main loop

;
;	Start memory test
;

Menu2:		
				ldx		#Text5						;	32 Kb memory test
				jsr		SendStringSCI

;				ldy		#SIZE_32K					;	Size
;				ldx		#START_32K					;	Start memory address
;				jsr		MemTest

				ldx		#Text6						;	8 Kb memory test
				jsr		SendStringSCI

				ldy		#SIZE_8K					;	Size
				ldx		#START_8K					;	Start memory address
				jsr		MemTest

				bra		Command						;	return to the main loop
					
*******************************************************************************************
*	void MemTest(x = Start address, y = Size memory)
*
*	Perform a complete memory test
*******************************************************************************************

MemTest:		psha

				ldaa	#DATA_ECRIT					;	A <= data to write

Loop:			cpy		#0							;	y < 0 ?
				beq		EndMemTest					;	Yes, so exit
				staa	0,X							;	Non, Mem[X] <= A
				ldaa	0,X							;	A <= Mem[X]
				cmpa	#DATA_ECRIT					;	A = data wrote ?
				bne		Error						;	Non, so error !

				pshx
				jsr		SendSCIAsciiX				;	Display the address error	
				ldx		#CRLF
				jsr		SendStringSCI		
				pulx

				inx									;	Next location
				dey									;	dec. number of location to do

				bra		Loop						;	Continue

Error:			pshx
				ldx		#Text8						;	Display the text error
				jsr		SendStringSCI
				pulx
				jsr		SendSCIAsciiX				;	Display the address error

				pula
				rts									;	End

EndMemTest:		ldx		#Text7						;	Successfull
				jsr		SendStringSCI
				
				pula
				rts									;	End

*******************************************************************************************
*	void InitSCI(void)
*
*	SCI initialisation.
*******************************************************************************************


InitSCI:		pshx
				ldx       #REGISTERS

				ldaa      #SCI_CONTROL				;	Initialise SCI port
				staa      SCCR2,X
				ldaa      #SCI_FORMAT				;	Set SCI data format
				staa      SCCR1,X
				ldaa      #SCI_BPS4800				;	Set SCI rate
				staa      BAUD,X

				pulx
				rts

*******************************************************************************************
*	void	SendString(X = @ chaine)
*
*	Send a string to the SCI until the END_STRING mark has been found.
*******************************************************************************************

 
SendStringSCI:	psha
				pshx
				
SendSCI:		ldaa      0,X          				;	Load byte to send	
				cmpa      #END_STRING				;	End of string ?
				beq       SendStringEnd				;	Yes, so exit

				jsr       SendByteSCI				;	

				inx
				bra       SendSCI

SendStringEnd:	pulx
				pula
				rts

*******************************************************************************************
*	void	SendByte(Accumulator A)
*
*	Send the contents of A accumulator to the SCI port.
*******************************************************************************************

 
SendByteSCI:	pshb
				pshx

				ldx	      #REGISTERS

wait:  			ldab      SCSR,X
         		bitb      #TC						;	Awaiting Data Register empty
          		beq       wait
          		staa	  SCDR,X

				pulx
          		pulb
          		rts
				
*******************************************************************************************
* 	A <= GetByteSCI(void)
*
* 	Look if a byte was received but don't await it.(if Z=0 then Byte received)
*******************************************************************************************


GetByteSCI:		pshb
				pshx

				ldx		  #REGISTERS

				ldab      SCSR,X							;	Status register
				bitb      #RDRF								;	byte received ?
				beq       EndGet							;	No

				ldaa      SCDR,X							;	A <= byte
				psha				
				tpa											;	A <= CCR
				anda      #CCR_BITZ							;	Bit Z <= 0
				tap											;	CCR <= A
				pula

EndGet:			
				pulx
				pulb
				rts

*******************************************************************************************
* 	A <= WaitByteSCI(void)
*
*	Wait a byte from the SCO port.	
*******************************************************************************************

WaitByteSCI:	jsr			GetByteSCI
				beq			WaitByteSCI
				rts

*******************************************************************************************
*	void SendAsciiX(X = valeur 16 bits )
*
*	Routine qui transmet le nombre contenu dans X au terminal sous la forme ASCII.
*******************************************************************************************
 
SendSCIAsciiX:	psha
				pshb
				pshx
				xgdx
				jsr			SendSCIAsciiA
				tba
				jsr			SendSCIAsciiA
				pulx
				pulb
				pula
				rts

*******************************************************************************************
*	void SendAsciiA(A = valeur 8 bits )
*
*	Routine qui transmet la valeur de A sur le terminal, sous la forme ASCII.
*******************************************************************************************
 
SendSCIAsciiA:	psha
				psha
				lsra
				lsra
				lsra
				lsra
				jsr			HexToAscii   					;	conversion
				jsr			SendByteSCI   					;	transmet le MSB
				pula
				anda		#$0F
				jsr			HexToAscii  					;	conversion
				jsr			SendByteSCI   					;	transmet le LSB
				pula
				rts

*******************************************************************************************
*	A <= HexToAscii(A = valeur numerique)
*
*	Routine de conversion Hexadecimal vers ASCII
*******************************************************************************************
 
HexToAscii:		adda		#$30
				cmpa		#'9'
				bls			EndHexToAscii
				adda		#7

EndHexToAscii:	rts

*******************************************************************************************
*			Start of data area
*******************************************************************************************


				DATA

;	Messages

Welcome:		dc.b		27,'E'
				dc.b		"Motorola/Atari 68HC11-F1 Memory Test program"
				dc.b		13,10
				dc.b		"+++++++ (c) 1996-97 France, FAVARD L. ++++++"
				dc.b		13,10
				dc.b		" (G) to start memory test"
				dc.b		13,10
				dc.b		" (?) about this program"
				dc.b		13,10,10,'>',' ',END_STRING

Text4:			dc.b		13,10
				dc.b		"Unknow command"
				dc.b		13,10,10,'>',' ',END_STRING

Text5:			dc.b		27,'E'
				dc.b		"STARTING MEMORY TEST FOR THE 32Kb"
				dc.b		13,10,END_STRING
				
Text6:			dc.b		"STARTING MEMORY TEST FOR THE 8Kb"
				dc.b		13,10,END_STRING
				
Text7:			dc.b		"Memory test all done!"
				dc.b		13,10,END_STRING
				
Text8:			dc.b		"Memory faillure at address : $"
				dc.b		END_STRING

CRLF:			dc.b		13,10,END_STRING

;	program 's stack location of 50 bytes

				ORG			$02FF
			
Stack:			dc.b		0								;	Top of internal SRAM
	
	
				End


